'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fs = _interopDefault(require('fs'));
var path = _interopDefault(require('path'));
var findCoffee = _interopDefault(require('find-coffee'));
var sake = require('sake-core');
var sake__default = _interopDefault(sake);
var yargs = _interopDefault(require('yargs'));

// src/transform.coffee
var transform;

var transform$1 = transform = function(code) {
  var babel, esModules, out;
  babel = require('babel-core');
  esModules = require('babel-plugin-transform-es2015-modules-commonjs');
  out = babel.transform(code, {
    plugins: [
      [
        esModules,
        {
          loose: true
        }
      ]
    ]
  });
  return out.code;
};

// src/cache.coffee
var cache, cacheDir, cachePath, findNodeModules, load, mkdirp, readOrigin, requireCached, write;

// Find local node_modules relative to current project
findNodeModules = function(dir) {
  var p;
  while (true) {
    p = path.join(dir, 'node_modules');
    if (fs.existsSync(p)) {
      return p;
    }
    if (dir === '/') {
      throw new Error('Unable to locate node_modules');
    }
    dir = path.resolve(path.join(dir, '..'));
  }
};

// Path to cache directory
cacheDir = function(dir) {
  return path.join(findNodeModules(dir), '.sake');
};

// Path to cache file
cachePath = function(dir) {
  return path.join(cacheDir(dir), 'Sakefile.js');
};

// Safely make directory
mkdirp = function(dir) {
  var err;
  try {
    return fs.mkdirSync(dir);
  } catch (error) {
    err = error;
    if (err.code !== 'EEXIST') {
      throw err;
    }
  }
};

// Read origin dir for cache file
readOrigin = function(dir) {
  return fs.readFileSync(path.join(cacheDir(dir), 'origin'), 'utf8');
};

// Write cache file
write = function(dir, code) {
  var transformed;
  mkdirp(cacheDir(dir));
  transformed = transform$1(code);
  fs.writeFileSync(cachePath(dir), transformed, 'utf8');
  return fs.writeFileSync(path.join(cacheDir(dir), 'origin'), dir, 'utf8');
};

// Require cached Sakefile such that require and other Node machinery work
requireCached = function(dir) {
  var err, tempFile;
  // Symlink cached Sakefile back into Sakefile dir
  tempFile = path.resolve(path.join(dir, "__Sakefile.js"));
  fs.linkSync(cachePath(dir), tempFile);
  try {
    require(tempFile); // Require cached Sakefile
  } catch (error) {
    err = error;
  }
  fs.unlinkSync(tempFile); // Ensure link is cleaned up
  if (err != null) {
    throw err; // Throw if failed to require Sakefile
  }
};


// Load cached Sakefile unless it's older than source or from different Sakefile
load = function(dir, file) {
  var cached, err, source;
  try {
    cached = fs.statSync(cachePath(dir));
    source = fs.statSync(path.join(dir, file));
  } catch (error) {
    err = error;
    if (err.code === 'ENOENT') {
      return false;
    }
    throw err;
  }
  if (cached.mtime > source.mtime && dir === (readOrigin(dir))) {
    requireCached(dir);
    return true;
  } else {
    return false;
  }
};

var cache$1 = cache = {
  load: load,
  write: write,
  require: requireCached
};

// src/load.coffee
var load$1, loadCakefile, loadSakefile, loadSakefileTs;

// Load Cakefile
loadCakefile = function(dir, file) {
  var code, coffee, js;
  // Try to load from cache
  if (cache$1.load(dir, file)) {
    return;
  }
  // Find local copy of CoffeeScript
  coffee = findCoffee();
  // Register .coffee extension
  coffee.register();
  // Compile Cakefile
  code = fs.readFileSync(file, 'utf8');
  js = coffee.compile(code, {
    bare: true,
    filename: file,
    sourceMap: false
  });
  // Write to cache and try to load
  cache$1.write(dir, js);
  return cache$1.require(dir);
};

// Load Sakefile
loadSakefile = function(dir, file) {
  var code, err;
  // Try to load from cache
  if (cache$1.load(dir, file)) {
    return;
  }
  try {
    code = fs.readFileSync(file, 'utf8');
    cache$1.write(dir, code);
    return cache$1.require(dir);
  } catch (error) {
    err = error;
    if ((path.extname(file)) !== '') {
      throw err;
    }
    if (err.constructor.name !== 'SyntaxError') {
      throw err;
    }
    return loadCakefile(dir, file);
  }
};

// Load Sakefile.ts
loadSakefileTs = function(dir, file) {
  // Try to load from cache
  if (cache$1.load(dir, file)) {
    return;
  }
  // Write straight to cache
  exec.sync("tsc --types sake-core --target ES6 --outFile .sake/Sakefile.js Sakefile.ts");
  // Load from cache
  return cache$1.require(dir);
};

var load$2 = load$1 = {
  Cakefile: loadCakefile,
  Sakefile: loadSakefile,
  SakefileTs: loadSakefileTs
};

// src/utils.coffee

// When `sake` is invoked, search in the current and all parent directories
// to find the relevant Sakefile (or Cakefile).
var findSakefile = function(dir) {
  var file, i, len, parent, ref;
  ref = ['Sakefile', 'Sakefile.js', 'Cakefile', 'Sakefile.ts'];
  for (i = 0, len = ref.length; i < len; i++) {
    file = ref[i];
    if (fs.existsSync(path.join(dir, file))) {
      return {
        file: file,
        dir: dir
      };
    }
  }
  // Recurse up the directory structure
  parent = path.normalize(path.join(dir, '..'));
  if (parent !== dir) {
    return findSakefile(parent);
  }
  // We're at the top, bail
  throw new Error(`Sakefile not found in ${process.cwd()}`);
};

// Display the list of Cake tasks in a format similar to `rake -T`
var printTasks = function(dir, file) {
  var desc, filePath, i, len, name, names, relative, results, spaces, task;
  relative = path.relative || path.resolve;
  filePath = path.join(relative(__originalDirname, process.cwd()), file);
  console.log(`${filePath} defines the following tasks:\n`);
  delete sake.tasks.has;
  names = Object.keys(sake.tasks).sort();
  results = [];
  for (i = 0, len = names.length; i < len; i++) {
    name = names[i];
    task = sake.tasks[name];
    spaces = 20 - name.length;
    spaces = spaces > 0 ? Array(spaces + 1).join(' ') : '';
    desc = task.description ? `# ${task.description}` : '';
    results.push(console.log(`sake ${name}${spaces} ${desc}`));
  }
  return results;
};

// console.log oparse.help() if switches.length

// Print an error and exit when attempting to use an invalid task/option.
var fatalError = function(message) {
  console.error(message + '\n');
  console.log('To see a list of all tasks/options, run "sake"');
  return process.exit(1);
};

var missingTask = function(task) {
  return fatalError(`No such task: ${task}`);
};

// package.json
var version = "0.6.2";

// src/run.coffee
var run;

// Run `sake`. Executes all of the tasks you pass, in order.  If no tasks are
// passed, print the help screen. Keep a reference to the original directory
// name, when running Cake tasks from subdirectories.
var run$1 = run = function() {
  var argv, dir, file, i, len, task, tasks;
  // Save record of original directory
  global.__originalDirname = fs.realpathSync('.');
  // Search for sakefile
  ({dir, file} = findSakefile(__originalDirname));
  // Change dir to match Sakefile location
  process.chdir(dir);
  // Process arguments
  argv = yargs.argv;
  tasks = argv._.slice(0); // Create a copy of tasks to execute
  argv.arguments = argv._; // For backwards compatibility with cake
  if (argv.debug) {
    process.env.VERBOSE = true;
  }
  // Print Sake version
  if (argv.version || argv.v && !argv._.length) {
    console.log(`${version} (core: ${sake__default.version})`);
    process.exit(0);
  }
  // Install Sake globals
  sake__default.install();
  // Load Sakefile
  if (!cache$1.load(dir, file)) {
    switch (file) {
      case 'Cakefile':
        load$2.Cakefile(dir, file);
        break;
      case 'Sakefile':
      case 'Sakefile.js':
        load$2.Sakefile(dir, file);
        break;
      case 'Sakefile.ts':
        load$2.SakefileTs(dir, file);
    }
  }
  if (!argv._.length) {
    // Bail if no tasks specified
    return printTasks(dir, file);
  }
// Bail if missing task
  for (i = 0, len = tasks.length; i < len; i++) {
    task = tasks[i];
    if (sake__default.tasks[task] == null) {
      missingTask(task);
    }
  }
  // Let's drink
  return sake__default.invoke(tasks, argv, function(err) {
    if (err != null) {
      console.error(err);
      return process.exit(1);
    }
  });
};

// src/cli.coffee

run$1();
//# sourceMappingURL=sake.js.map
